@@ -40,6 +40,7 @@ from picard.coverart.providers import (
     CoverArtProvider,
     cover_art_providers,
 )
+from picard.metadata import register_album_metadata_processor
 
 
 class CoverArt:
@@ -72,9 +73,9 @@ class CoverArt:
                     coverartimage,
                     coverartimage.imageinfo_as_string())
                 )
-                self.metadata.append_image(coverartimage)
+                self.metadata.images.append(coverartimage)
                 for track in self.album._new_tracks:
-                    track.metadata.append_image(coverartimage)
+                    track.metadata.images.append(coverartimage)
                 # If the image already was a front image,
                 # there might still be some other non-CAA front
                 # images in the queue - ignore them.
@@ -127,28 +128,28 @@ class CoverArt:
             # album removed
             return
 
-        if (self.front_image_found and
-                config.setting["save_images_to_tags"] and not
-                config.setting["save_images_to_files"] and
-                config.setting["embed_only_one_front_image"]):
+        if (self.front_image_found
+            and config.setting["save_images_to_tags"]
+            and not config.setting["save_images_to_files"]
+            and config.setting["embed_only_one_front_image"]):
             # no need to continue
             self.album._finalize_loading(None)
             return
 
         if self._queue_empty():
-            if self.providers:
+            try:
                 # requeue from next provider
-                provider = self.providers.pop(0)
+                provider = next(self.providers)
                 ret = CoverArtProvider._STARTED
                 try:
-                    p = provider(self)
-                    if p.enabled():
+                    instance = provider.cls(self)
+                    if provider.enabled and instance.enabled():
                         log.debug("Trying cover art provider %s ..." %
-                                  provider.NAME)
-                        ret = p.queue_images()
+                                  provider.name)
+                        ret = instance.queue_images()
                     else:
                         log.debug("Skipping cover art provider %s ..." %
-                                  provider.NAME)
+                                  provider.name)
                 except BaseException:
                     log.error(traceback.format_exc())
                     raise
@@ -156,7 +157,7 @@ class CoverArt:
                     if ret != CoverArtProvider.WAIT:
                         self.next_in_queue()
                     return
-            else:
+            except StopIteration:
                 # nothing more to do
                 self.album._finalize_loading(None)
                 return
@@ -172,14 +173,15 @@ class CoverArt:
             return
 
         # local files
-        if hasattr(coverartimage, 'filepath'):
+        if coverartimage.url and coverartimage.url.scheme() == 'file':
             try:
-                with open(coverartimage.filepath, 'rb') as file:
+                path = coverartimage.url.toLocalFile()
+                with open(path, 'rb') as file:
                     self._set_metadata(coverartimage, file.read())
             except IOError as ioexcept:
                 (errnum, errmsg) = ioexcept.args
                 log.error("Failed to read %r: %s (%d)" %
-                          (coverartimage.from_file, errmsg, errnum))
+                          (path, errmsg, errnum))
             except CoverArtImageIOError:
                 # It doesn't make sense to store/download more images if we can't
                 # save them in the temporary folder, abort.
@@ -230,10 +232,13 @@ class CoverArt:
         QObject.tagger.window.set_statusbar_message(*args, **kwargs)
 
 
-def coverart(album, metadata, release):
+def _retrieve_coverart(album, metadata, release):
     """Gets all cover art URLs from the metadata and then attempts to
     download the album art. """
 
     coverart = CoverArt(album, metadata, release)
     log.debug("New %r", coverart)
     coverart.retrieve()
+
+
+register_album_metadata_processor(_retrieve_coverart)
